# This is going to sort out what the model predicts the probabily of red will be based on each possible combination of the task.

from pymc import *
import scipy.stats
import numpy as np
from tabulate import tabulate
import csv

def make_model(subsees = 4, outof = 8, nothers = 4, otherschoose = [0,0,1,1],othersconf=[1,1,1,1], confk = 5, othersseeupto = 8):
    # subsees: number of red subject sees
    # outof: total number of draws subject sees
    # nothers: number of other players
    # otherchoose: what they choose (1/red, 0/blue)
    # othersseeupto: how many draws they might have seen at max
    # othersrt: their RTs, right now -0.7 is confidient and -0.4 is not
    
    # the probability of red
    PJ = Uniform('PJ',0,1)
    
    # the number of red observed by the subject
    Sample = Binomial('Sample',outof,PJ,observed=True,value=subsees)

    # total number of marbles observed by each opponent (same for everbody)
    N = DiscreteUniform('N',1,othersseeupto)
    #N=10

    # number of red observed by each opponent
    AS= [Binomial('OOR_%i'%i,N,PJ) for i in xrange(nothers)]
    
    # the real confidence of each opponent (-.5 confident blue, .5 confident red)
    # this lambda business is working around a weird bug
    RC = [pymc.Lambda('RC_%i'%i,lambda red=AS[i],num=N: scipy.stats.beta.cdf(0.5,num-red+1,red+1)-.5)  for i in xrange(nothers)]

    # probability that each opponent responds red
    PAR = [InvLogit('PAR_%i'%i,10*RC[i]) for i in xrange(nothers)]
   
    # each opponent's answer (0 or 1 for blue or red)
    AR = [Bernoulli('AR_%i'%i,PAR[i],observed=True,value=otherschoose[i]) for i in xrange(nothers)]

    # probability that each opponent answers confident
    PACON= [InvLogit('PACON_%i'%i, confk*(abs(RC[i]) - 0.25)) for i in xrange(nothers)]

    # each opponent's confidence  answer (0 for unconfident, 1 for confident)
    ACON = [Bernoulli('RC_%i'%i,PACON[i],observed=True,value=othersconf[i]) for i in xrange(nothers)]
    return locals()
    
   
   # In the behavioral task, there are limited possibilites. Each person that chooses the same colour has the same confidence level. So there are limited numbers of combinations.  
   # Also, there is a range of 1 to 7 marbles in the sample.  
   # In Oconfi (below), each value represents a single agent, and these are grouped into 4. 0 refers to low confidence, and 1 refers to confident.

#              0         1          2        3        4
OConfi =  [[0,0,0,0],[1,0,0,0],[1,1,0,0],[1,1,1,0],[1,1,1,1]]

L=[]
OConfi =  []  
for nred in range(1,8):
        
        # 0 Red Choices
        OConfi =  [[0,0,0,0],[1,1,1,1]]
        RC = [0,0] # For reporting (in list L) the confidence of the red choosing agents.
        GC = [0,1] # For reporting (in list L) the confidence of the green choosing agents.     
        for confiset in range(0,2): 
                S=MCMC(make_model(subsees=nred, otherschoose = [0,0,0,0], othersconf=OConfi[confiset],confk = 10))
                S.sample(iter=10000,thin=5,burn=5000,tune_interval=250,tune_throughout=False,  verbose=0)
                L.append([np.median(S.PJ.trace[:]), nred, 0, RC[confiset], GC[confiset]])
        
        # 1 Red Choice
        OConfi = [[0,0,0,0],[1,0,0,0],[1,1,1,1],[0,1,1,1]]
        RC = [0,1,1,0]
        GC = [0,0,1,1]     
        for confiset in range(0,4):
                S=MCMC(make_model(subsees=nred, otherschoose = [1,0,0,0], othersconf=OConfi[confiset],confk = 10))
                S.sample(iter=10000,thin=5,burn=5000,tune_interval=250,tune_throughout=False,  verbose=0)
                L.append([np.median(S.PJ.trace[:]),nred, 1, RC[confiset], GC[confiset]])
        
        #2 Red Choices
        RC = [0,1,1,0]
        GC = [0,0,1,1] 
        OConfi = [[0,0,0,0],[1,1,0,0],[1,1,1,1],[0,0,1,1]]    
        for confiset in range(0,4): 
                S=MCMC(make_model(subsees=nred, otherschoose = [1,1,0,0], othersconf=OConfi[confiset],confk = 10))
                S.sample(iter=10000,thin=5,burn=5000,tune_interval=250,tune_throughout=False,  verbose=0)
                L.append([np.median(S.PJ.trace[:]),nred, 2, RC[confiset], GC[confiset]])
        
        # 3 Red Choice
        RC = [0,1,1,0]
        GC = [0,0,1,1] 
        OConfi = [[0,0,0,0],[1,1,1,0],[1,1,1,1],[0,0,0,1]]        
        for confiset in range(0,4): 
                S=MCMC(make_model(subsees=nred, otherschoose = [1,1,1,0], othersconf=OConfi[confiset],confk = 10))
                S.sample(iter=10000,thin=5,burn=5000,tune_interval=250,tune_throughout=False,  verbose=0)
                L.append([np.median(S.PJ.trace[:]),nred, 3, RC[confiset], GC[confiset]])
        
        #4 Red Choices
        RC = [0,1]
        GC = [0,0] 
        OConfi = [[0,0,0,0],[1,1,1,1]]        
        for confiset in range(0,2): 
                S=MCMC(make_model(subsees=nred, otherschoose = [1,1,1,1], othersconf=OConfi[confiset],confk = 10))
                S.sample(iter=10000,thin=5,burn=5000,tune_interval=250,tune_throughout=False,  verbose=0)
                L.append([np.median(S.PJ.trace[:]),nred, 4, RC[confiset], GC[confiset]])
    
print tabulate(L, headers=["rProbability","nRed","OthersChooseRed", "RedConfidence","GreenConfidence"])
                
                          
      